@@ -8,14 +8,15 @@
 
 """Common objects shared by all _ps* modules."""
 
-from psutil._compat import namedtuple
+from __future__ import division
+from psutil._compat import namedtuple, long
 
 # --- functions
 
 def usage_percent(used, total, _round=None):
     """Calculate percentage usage of 'used' against 'total'."""
     try:
-        ret = (float(used) / total) * 100
+        ret = (used / total) * 100
     except ZeroDivisionError:
         ret = 0
     if _round is not None:
@@ -37,6 +38,44 @@ class constant(int):
     def __str__(self):
         return self._name
 
+    def __eq__(self, other):
+        # Use both int or str values when comparing for equality
+        # (useful for serialization):
+        # >>> st = constant(0, "running")
+        # >>> st == 0
+        # True
+        # >>> st == 'running'
+        # True
+        if isinstance(other, int):
+            return int(self) == other
+        if isinstance(other, long):
+            return long(self) == other
+        if isinstance(other, str):
+            return self._name == other
+        return False
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+def memoize(f):
+    """A simple memoize decorator for functions."""
+    cache= {}
+    def memf(*x):
+        if x not in cache:
+            cache[x] = f(*x)
+        return cache[x]
+    return memf
+
+class cached_property(object):
+    """A memoize decorator for class properties."""
+    def __init__(self, func):
+        self.func = func
+
+    def __get__(self, instance, type):
+        res = instance.__dict__[self.func.__name__] = self.func(instance)
+        return res
+
+
 # --- constants
 
 STATUS_RUNNING = constant(0, "running")
@@ -80,22 +119,21 @@ del AF_INET, AF_INET6, SOCK_STREAM, SOCK_DGRAM, socket
 # --- namedtuples
 
 # system
-ntuple_sys_cputimes = namedtuple('cputimes', 'user nice system idle iowait irq softirq')
-ntuple_sysmeminfo = namedtuple('usage', 'total used free percent')
-ntuple_diskinfo = namedtuple('usage', 'total used free percent')
-ntuple_partition = namedtuple('partition',  'device mountpoint fstype')
-ntuple_net_iostat = namedtuple('iostat', 'bytes_sent bytes_recv packets_sent packets_recv')
-ntuple_disk_iostat = namedtuple('iostat', 'read_count write_count read_bytes write_bytes read_time write_time')
-ntuple_user = namedtuple('user', 'name terminal host started user_process')
-
+nt_sys_cputimes = namedtuple('cputimes', 'user nice system idle iowait irq softirq')
+nt_sysmeminfo = namedtuple('usage', 'total used free percent')
+nt_diskinfo = namedtuple('usage', 'total used free percent')
+nt_partition = namedtuple('partition',  'device mountpoint fstype opts')
+nt_net_iostat = namedtuple('iostat', 'bytes_sent bytes_recv packets_sent packets_recv')
+nt_disk_iostat = namedtuple('iostat', 'read_count write_count read_bytes write_bytes read_time write_time')
+nt_user = namedtuple('user', 'name terminal host started')
 
 # processes
-ntuple_meminfo = namedtuple('meminfo', 'rss vms')
-ntuple_cputimes = namedtuple('cputimes', 'user system')
-ntuple_openfile = namedtuple('openfile', 'path fd')
-ntuple_connection = namedtuple('connection', 'fd family type local_address remote_address status')
-ntuple_thread = namedtuple('thread', 'id user_time system_time')
-ntuple_uids = namedtuple('user', 'real effective saved')
-ntuple_gids = namedtuple('group', 'real effective saved')
-ntuple_io = namedtuple('io', 'read_count write_count read_bytes write_bytes')
-ntuple_ionice = namedtuple('ionice', 'ioclass value')
+nt_meminfo = namedtuple('meminfo', 'rss vms')
+nt_cputimes = namedtuple('cputimes', 'user system')
+nt_openfile = namedtuple('openfile', 'path fd')
+nt_connection = namedtuple('connection', 'fd family type local_address remote_address status')
+nt_thread = namedtuple('thread', 'id user_time system_time')
+nt_uids = namedtuple('user', 'real effective saved')
+nt_gids = namedtuple('group', 'real effective saved')
+nt_io = namedtuple('io', 'read_count write_count read_bytes write_bytes')
+nt_ionice = namedtuple('ionice', 'ioclass value')
