@@ -10,10 +10,11 @@
 
 import errno
 import os
+import sys
 
 import _psutil_bsd
 import _psutil_posix
-import _psposix
+from psutil import _psposix
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._compat import namedtuple
 from psutil._common import *
@@ -36,7 +37,7 @@ def phymem_usage():
     used = total - free
     # XXX check out whether we have to do the same math we do on Linux
     percent = usage_percent(used, total, _round=1)
-    return ntuple_sysmeminfo(total, used, free, percent)
+    return nt_sysmeminfo(total, used, free, percent)
 
 def virtmem_usage():
     """Virtual system memory as a (total, used, free) tuple."""
@@ -44,7 +45,7 @@ def virtmem_usage():
     free =  _psutil_bsd.get_avail_virtmem()
     used = total - free
     percent = usage_percent(used, total, _round=1)
-    return ntuple_sysmeminfo(total, used, free, percent)
+    return nt_sysmeminfo(total, used, free, percent)
 
 def get_system_cpu_times():
     """Return system per-CPU times as a named tuple"""
@@ -81,17 +82,28 @@ def disk_partitions(all=False):
     retlist = []
     partitions = _psutil_bsd.get_disk_partitions()
     for partition in partitions:
-        device, mountpoint, fstype = partition
+        device, mountpoint, fstype, opts = partition
         if device == 'none':
             device = ''
         if not all:
             if not os.path.isabs(device) \
             or not os.path.exists(device):
                 continue
-        ntuple = ntuple_partition(device, mountpoint, fstype)
+        ntuple = nt_partition(device, mountpoint, fstype, opts)
         retlist.append(ntuple)
     return retlist
 
+def get_system_users():
+    retlist = []
+    rawlist = _psutil_bsd.get_system_users()
+    for item in rawlist:
+        user, tty, hostname, tstamp = item
+        if tty == '~':
+            continue  # reboot or shutdown
+        nt = nt_user(user, tty or None, hostname, tstamp)
+        retlist.append(nt)
+    return retlist
+
 get_pid_list = _psutil_bsd.get_pid_list
 pid_exists = _psposix.pid_exists
 get_disk_usage = _psposix.get_disk_usage
@@ -107,7 +119,8 @@ def wrap_exceptions(method):
     def wrapper(self, *args, **kwargs):
         try:
             return method(self, *args, **kwargs)
-        except OSError, err:
+        except OSError:
+            err = sys.exc_info()[1]
             if err.errno == errno.ESRCH:
                 raise NoSuchProcess(self.pid, self._process_name)
             if err.errno in (errno.EPERM, errno.EACCES):
@@ -168,31 +181,33 @@ class Process(object):
         @wrap_exceptions
         def get_process_cwd(self):
             """Return process current working directory."""
-            return _psutil_bsd.get_process_cwd(self.pid)
+            # sometimes we get an empty string, in which case we turn
+            # it into None
+            return _psutil_bsd.get_process_cwd(self.pid) or None
 
     @wrap_exceptions
     def get_process_uids(self):
         """Return real, effective and saved user ids."""
         real, effective, saved = _psutil_bsd.get_process_uids(self.pid)
-        return ntuple_uids(real, effective, saved)
+        return nt_uids(real, effective, saved)
 
     @wrap_exceptions
     def get_process_gids(self):
         """Return real, effective and saved group ids."""
         real, effective, saved = _psutil_bsd.get_process_gids(self.pid)
-        return ntuple_gids(real, effective, saved)
+        return nt_gids(real, effective, saved)
 
     @wrap_exceptions
     def get_cpu_times(self):
         """return a tuple containing process user/kernel time."""
         user, system = _psutil_bsd.get_cpu_times(self.pid)
-        return ntuple_cputimes(user, system)
+        return nt_cputimes(user, system)
 
     @wrap_exceptions
     def get_memory_info(self):
         """Return a tuple with the process' RSS and VMS size."""
         rss, vms = _psutil_bsd.get_memory_info(self.pid)
-        return ntuple_meminfo(rss, vms)
+        return nt_meminfo(rss, vms)
 
     @wrap_exceptions
     def get_process_create_time(self):
@@ -205,13 +220,18 @@ class Process(object):
         """Return the number of threads belonging to the process."""
         return _psutil_bsd.get_process_num_threads(self.pid)
 
+    @wrap_exceptions
+    def get_num_fds(self):
+        """Return the number of file descriptors opened by this process."""
+        return _psutil_bsd.get_process_num_fds(self.pid)
+
     @wrap_exceptions
     def get_process_threads(self):
         """Return the number of threads belonging to the process."""
         rawlist = _psutil_bsd.get_process_threads(self.pid)
         retlist = []
         for thread_id, utime, stime in rawlist:
-            ntuple = ntuple_thread(thread_id, utime, stime)
+            ntuple = nt_thread(thread_id, utime, stime)
             retlist.append(ntuple)
         return retlist
 
@@ -222,7 +242,7 @@ class Process(object):
         # else fallback on lsof parser
         if hasattr(_psutil_bsd, "get_process_open_files"):
             rawlist = _psutil_bsd.get_process_open_files(self.pid)
-            return [ntuple_openfile(path, fd) for path, fd in rawlist]
+            return [nt_openfile(path, fd) for path, fd in rawlist]
         else:
             lsof = _psposix.LsofParser(self.pid, self._process_name)
             return lsof.get_process_open_files()
@@ -267,4 +287,22 @@ class Process(object):
     @wrap_exceptions
     def get_process_io_counters(self):
         rc, wc, rb, wb = _psutil_bsd.get_process_io_counters(self.pid)
-        return ntuple_io(rc, wc, rb, wb)
+        return nt_io(rc, wc, rb, wb)
+
+    nt_mmap_grouped = namedtuple('mmap',
+        'path rss, private, ref_count, shadow_count')
+    nt_mmap_ext = namedtuple('mmap',
+        'addr, perms path rss, private, ref_count, shadow_count')
+
+    @wrap_exceptions
+    def get_memory_maps(self):
+        return _psutil_bsd.get_process_memory_maps(self.pid)
+
+    # FreeBSD < 8 does not support kinfo_getfile() and kinfo_getvmmap()
+    if not hasattr(_psutil_bsd, 'get_process_open_files'):
+        def _not_implemented(self):
+            raise NotImplementedError("supported only starting from FreeBSD 8")
+        get_open_files = _not_implemented
+        get_process_cwd = _not_implemented
+        get_memory_maps = _not_implemented
+        get_num_fds = _not_implemented
