@@ -10,10 +10,11 @@
 
 import errno
 import os
+import sys
 
 import _psutil_osx
 import _psutil_posix
-import _psposix
+from psutil import _psposix
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._compat import namedtuple
 from psutil._common import *
@@ -35,7 +36,7 @@ def phymem_usage():
     free =  _psutil_osx.get_avail_phymem()
     used = total - free
     percent = usage_percent(used, total, _round=1)
-    return ntuple_sysmeminfo(total, used, free, percent)
+    return nt_sysmeminfo(total, used, free, percent)
 
 def virtmem_usage():
     """Virtual system memory as a (total, used, free) tuple."""
@@ -43,7 +44,7 @@ def virtmem_usage():
     free =  _psutil_osx.get_avail_virtmem()
     used = total - free
     percent = usage_percent(used, total, _round=1)
-    return ntuple_sysmeminfo(total, used, free, percent)
+    return nt_sysmeminfo(total, used, free, percent)
 
 def get_system_cpu_times():
     """Return system CPU times as a namedtuple."""
@@ -63,17 +64,31 @@ def disk_partitions(all=False):
     retlist = []
     partitions = _psutil_osx.get_disk_partitions()
     for partition in partitions:
-        device, mountpoint, fstype = partition
+        device, mountpoint, fstype, opts = partition
         if device == 'none':
             device = ''
         if not all:
             if not os.path.isabs(device) \
             or not os.path.exists(device):
                 continue
-        ntuple = ntuple_partition(device, mountpoint, fstype)
+        ntuple = nt_partition(device, mountpoint, fstype, opts)
         retlist.append(ntuple)
     return retlist
 
+def get_system_users():
+    retlist = []
+    rawlist = _psutil_osx.get_system_users()
+    for item in rawlist:
+        user, tty, hostname, tstamp = item
+        if tty == '~':
+            continue  # reboot or shutdown
+        if not tstamp:
+            continue
+        nt = nt_user(user, tty or None, hostname or None, tstamp)
+        retlist.append(nt)
+    return retlist
+
+
 get_pid_list = _psutil_osx.get_pid_list
 pid_exists = _psposix.pid_exists
 get_disk_usage = _psposix.get_disk_usage
@@ -90,7 +105,8 @@ def wrap_exceptions(callable):
     def wrapper(self, *args, **kwargs):
         try:
             return callable(self, *args, **kwargs)
-        except OSError, err:
+        except OSError:
+            err = sys.exc_info()[1]
             if err.errno == errno.ESRCH:
                 raise NoSuchProcess(self.pid, self._process_name)
             if err.errno in (errno.EPERM, errno.EACCES):
@@ -143,12 +159,12 @@ class Process(object):
     @wrap_exceptions
     def get_process_uids(self):
         real, effective, saved = _psutil_osx.get_process_uids(self.pid)
-        return ntuple_uids(real, effective, saved)
+        return nt_uids(real, effective, saved)
 
     @wrap_exceptions
     def get_process_gids(self):
         real, effective, saved = _psutil_osx.get_process_gids(self.pid)
-        return ntuple_gids(real, effective, saved)
+        return nt_gids(real, effective, saved)
 
     @wrap_exceptions
     def get_process_terminal(self):
@@ -162,12 +178,12 @@ class Process(object):
     def get_memory_info(self):
         """Return a tuple with the process' RSS and VMS size."""
         rss, vms = _psutil_osx.get_memory_info(self.pid)
-        return ntuple_meminfo(rss, vms)
+        return nt_meminfo(rss, vms)
 
     @wrap_exceptions
     def get_cpu_times(self):
         user, system = _psutil_osx.get_cpu_times(self.pid)
-        return ntuple_cputimes(user, system)
+        return nt_cputimes(user, system)
 
     @wrap_exceptions
     def get_process_create_time(self):
@@ -184,12 +200,12 @@ class Process(object):
     def get_open_files(self):
         """Return files opened by process."""
         if self.pid == 0:
-            raise AccessDenied(self.pid, self._process_name)
+            return []
         files = []
         rawlist = _psutil_osx.get_process_open_files(self.pid)
         for path, fd in rawlist:
             if os.path.isfile(path):
-                ntuple = ntuple_openfile(path, fd)
+                ntuple = nt_openfile(path, fd)
                 files.append(ntuple)
         return files
 
@@ -203,7 +219,13 @@ class Process(object):
                              % (kind, ', '.join([repr(x) for x in conn_tmap])))
         families, types = conn_tmap[kind]
         ret = _psutil_osx.get_process_connections(self.pid, families, types)
-        return [ntuple_connection(*conn) for conn in ret]
+        return [nt_connection(*conn) for conn in ret]
+
+    @wrap_exceptions
+    def get_num_fds(self):
+        if self.pid == 0:
+            return 0
+        return _psutil_osx.get_process_num_fds(self.pid)
 
     @wrap_exceptions
     def process_wait(self, timeout=None):
@@ -233,6 +255,15 @@ class Process(object):
         rawlist = _psutil_osx.get_process_threads(self.pid)
         retlist = []
         for thread_id, utime, stime in rawlist:
-            ntuple = ntuple_thread(thread_id, utime, stime)
+            ntuple = nt_thread(thread_id, utime, stime)
             retlist.append(ntuple)
         return retlist
+
+    nt_mmap_grouped = namedtuple('mmap',
+        'path rss private swapped dirtied ref_count shadow_depth')
+    nt_mmap_ext = namedtuple('mmap',
+        'addr perms path rss private swapped dirtied ref_count shadow_depth')
+
+    @wrap_exceptions
+    def get_memory_maps(self):
+        return _psutil_osx.get_process_memory_maps(self.pid)
