@@ -14,16 +14,17 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <signal.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #include <sys/param.h>
 #include <sys/user.h>
 #include <sys/proc.h>
 #include <sys/socket.h>
-#include <fcntl.h>
+#include <utmp.h>         /* system users */
 #include <devstat.h>      /* get io counters */
 #include <sys/vmmeter.h>  /* needed for vmtotal struct */
-#include <libutil.h>      /* process open files */
+#include <libutil.h>      /* process open files, shared libs (kinfo_getvmmap) */
 #include <sys/mount.h>
 
 #include <net/if.h>       /* net io counters */
@@ -666,15 +667,13 @@ static PyObject*
 get_process_open_files(PyObject* self, PyObject* args)
 {
     long pid;
+    int i, cnt;
     PyObject *retList = PyList_New(0);
     PyObject *tuple = NULL;
 
     struct kinfo_file *freep, *kif;
     struct kinfo_proc kipp;
 
-
-    int i, cnt;
-
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
     if (get_kinfo_proc(pid, &kipp) == -1)
@@ -702,6 +701,33 @@ get_process_open_files(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return files opened by process as a list of (path, fd) tuples
+ */
+static PyObject*
+get_process_num_fds(PyObject* self, PyObject* args)
+{
+    long pid;
+    int cnt;
+
+    struct kinfo_file *freep, *kif;
+    struct kinfo_proc kipp;
+
+    if (! PyArg_ParseTuple(args, "l", &pid))
+        return NULL;
+    if (get_kinfo_proc(pid, &kipp) == -1)
+        return NULL;
+
+    freep = kinfo_getfile(pid, &cnt);
+    if (freep == NULL) {
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    return Py_BuildValue("i", cnt);
+}
+
+
 /*
  * Return process current working directory.
  */
@@ -799,6 +825,106 @@ get_system_per_cpu_times(PyObject* self, PyObject* args)
 
     return py_retlist;
 }
+
+
+/*
+ * Return a list of tuples for every process memory maps.
+ * 'procstat' cmdline utility has been used as an example.
+ */
+static PyObject*
+get_process_memory_maps(PyObject* self, PyObject* args)
+{
+    long pid;
+    int ptrwidth;
+    int i, cnt;
+    char addr[30];
+    char perms[10];
+    const char *path;
+    struct kinfo_proc kp;
+    struct kinfo_vmentry *freep, *kve;
+    PyObject* pytuple = NULL;
+    PyObject* retlist = PyList_New(0);
+
+    ptrwidth = 2*sizeof(void *);
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+
+    freep = kinfo_getvmmap(pid, &cnt);
+    if (freep == NULL) {
+        PyErr_SetString(PyExc_RuntimeError, "kinfo_getvmmap() failed");
+        return NULL;
+    }
+    for (i = 0; i < cnt; i++) {
+        kve = &freep[i];
+        addr[0] = '\0';
+        perms[0] = '\0';
+        sprintf(addr, "%#*jx-%#*jx", ptrwidth, (uintmax_t)kve->kve_start,
+                                     ptrwidth, (uintmax_t)kve->kve_end);
+        strlcat(perms, kve->kve_protection & KVME_PROT_READ ? "r" : "-",
+                sizeof(perms));
+        strlcat(perms, kve->kve_protection & KVME_PROT_WRITE ? "w" : "-",
+                sizeof(perms));
+        strlcat(perms, kve->kve_protection & KVME_PROT_EXEC ? "x" : "-",
+                sizeof(perms));
+
+        if (strlen(kve->kve_path) == 0) {
+            switch (kve->kve_type) {
+            case KVME_TYPE_NONE:
+                path = "[none]";
+                break;
+            case KVME_TYPE_DEFAULT:
+                path = "[default]";
+                break;
+            case KVME_TYPE_VNODE:
+                path = "[vnode]";
+                break;
+            case KVME_TYPE_SWAP:
+                path = "[swap]";
+                break;
+            case KVME_TYPE_DEVICE:
+                path = "[device]";
+                break;
+            case KVME_TYPE_PHYS:
+                path = "[phys]";
+                break;
+            case KVME_TYPE_DEAD:
+                path = "[dead]";
+                break;
+            case KVME_TYPE_SG:
+                path = "[sg]";
+                break;
+            case KVME_TYPE_UNKNOWN:
+                path = "[unknown]";
+                break;
+            default:
+                path = "[?]";
+                break;
+            }
+        }
+        else {
+            path = kve->kve_path;
+        }
+
+        pytuple = Py_BuildValue("sssiiii",
+            addr,                       // "start-end" address
+            perms,                      // "rwx" permissions
+            path,                       // path
+            kve->kve_resident,          // rss
+            kve->kve_private_resident,  // private
+            kve->kve_ref_count,         // ref count
+            kve->kve_shadow_count       // shadow count
+        );
+        PyList_Append(retlist, pytuple);
+        Py_XDECREF(pytuple);
+    }
+    free(freep);
+    return retlist;
+}
 #endif
 
 
@@ -812,6 +938,8 @@ get_disk_partitions(PyObject* self, PyObject* args)
     int num;
     int i;
     long len;
+    uint64_t flags;
+    char opts[200];
     struct statfs *fs;
     PyObject* py_retlist = PyList_New(0);
     PyObject* py_tuple;
@@ -838,9 +966,49 @@ get_disk_partitions(PyObject* self, PyObject* args)
     }
 
     for (i = 0; i < num; i++) {
-        py_tuple = Py_BuildValue("(sss)", fs[i].f_mntfromname,  // device
-                                          fs[i].f_mntonname,    // mount point
-                                          fs[i].f_fstypename);  // fs type
+        opts[0] = 0;
+        flags = fs[i].f_flags;
+
+        // see sys/mount.h
+        if (flags & MNT_RDONLY)
+            strlcat(opts, "ro", sizeof(opts));
+        else
+            strlcat(opts, "rw", sizeof(opts));
+        if (flags & MNT_SYNCHRONOUS)
+            strlcat(opts, ",sync", sizeof(opts));
+        if (flags & MNT_NOEXEC)
+            strlcat(opts, ",noexec", sizeof(opts));
+        if (flags & MNT_NOSUID)
+            strlcat(opts, ",nosuid", sizeof(opts));
+        if (flags & MNT_UNION)
+            strlcat(opts, ",union", sizeof(opts));
+        if (flags & MNT_ASYNC)
+            strlcat(opts, ",async", sizeof(opts));
+        if (flags & MNT_SUIDDIR)
+            strlcat(opts, ",suiddir", sizeof(opts));
+        if (flags & MNT_SOFTDEP)
+            strlcat(opts, ",softdep", sizeof(opts));
+        if (flags & MNT_NOSYMFOLLOW)
+            strlcat(opts, ",nosymfollow", sizeof(opts));
+        if (flags & MNT_GJOURNAL)
+            strlcat(opts, ",gjournal", sizeof(opts));
+        if (flags & MNT_MULTILABEL)
+            strlcat(opts, ",multilabel", sizeof(opts));
+        if (flags & MNT_ACLS)
+            strlcat(opts, ",acls", sizeof(opts));
+        if (flags & MNT_NOATIME)
+            strlcat(opts, ",noatime", sizeof(opts));
+        if (flags & MNT_NOCLUSTERR)
+            strlcat(opts, ",noclusterr", sizeof(opts));
+        if (flags & MNT_NOCLUSTERW)
+            strlcat(opts, ",noclusterw", sizeof(opts));
+        if (flags & MNT_NFS4ACLS)
+            strlcat(opts, ",nfs4acls", sizeof(opts));
+
+        py_tuple = Py_BuildValue("(ssss)", fs[i].f_mntfromname,  // device
+                                           fs[i].f_mntonname,    // mount point
+                                           fs[i].f_fstypename,   // fs type
+                                           opts);                // options
         PyList_Append(py_retlist, py_tuple);
         Py_XDECREF(py_tuple);
     }
@@ -980,6 +1148,41 @@ get_disk_io_counters(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return currently connected users as a list of tuples.
+ */
+static PyObject*
+get_system_users(PyObject* self, PyObject* args)
+{
+    PyObject *ret_list = PyList_New(0);
+    PyObject *tuple = NULL;
+    struct utmp ut;
+    FILE *fp;
+
+
+    fp = fopen(_PATH_UTMP, "r");
+    if (fp == NULL) {
+        return PyErr_SetFromErrno(0);
+    }
+
+    while (fread(&ut, sizeof(ut), 1, fp) == 1) {
+        if (*ut.ut_name == '\0')
+            continue;
+        tuple = Py_BuildValue("(sssf)",
+            ut.ut_name,              // username
+            ut.ut_line,              // tty
+            ut.ut_host,              // hostname
+            (float)ut.ut_time        // tstamp
+        );
+        PyList_Append(ret_list, tuple);
+        Py_DECREF(tuple);
+    }
+
+    fclose(fp);
+    return ret_list;
+}
+
+
 /*
  * define the psutil C module methods and initialize the module.
  */
@@ -1022,6 +1225,10 @@ PsutilMethods[] =
          "Return files opened by process as a list of (path, fd) tuples"},
      {"get_process_cwd", get_process_cwd, METH_VARARGS,
          "Return process current working directory."},
+     {"get_process_memory_maps", get_process_memory_maps, METH_VARARGS,
+         "Return a list of tuples for every process's memory map"},
+     {"get_process_num_fds", get_process_num_fds, METH_VARARGS,
+         "Return the number of file descriptors opened by this process"},
 #endif
 
      // --- system-related functions
@@ -1054,6 +1261,8 @@ PsutilMethods[] =
          "Return dict of tuples of networks I/O information."},
      {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
          "Return a Python dict of tuples for disk I/O information"},
+     {"get_system_users", get_system_users, METH_VARARGS,
+        "Return currently connected users as a list of tuples"},
 
      {NULL, NULL, 0, NULL}
 };
