@@ -13,6 +13,7 @@
 #include <stdlib.h>
 #include <mntent.h>
 #include <utmp.h>
+#include <sched.h>
 #include <sys/syscall.h>
 #include <sys/sysinfo.h>
 #include <linux/unistd.h>
@@ -119,9 +120,11 @@ get_disk_partitions(PyObject* self, PyObject* args)
             endmntent(file);
             return PyErr_Format(PyExc_RuntimeError, "getmntent() failed");
         }
-        py_tuple = Py_BuildValue("(sss)", entry->mnt_fsname,  // device
-                                          entry->mnt_dir,     // mount point
-                                          entry->mnt_type);   // fs type
+        py_tuple = Py_BuildValue("(ssss)", entry->mnt_fsname,  // device
+                                           entry->mnt_dir,     // mount point
+                                           entry->mnt_type,    // fs type
+                                           entry->mnt_opts);   // options
+
         PyList_Append(py_retlist, py_tuple);
         Py_XDECREF(py_tuple);
     }
@@ -148,6 +151,80 @@ get_physmem(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return process CPU affinity as a Python long (the bitmask)
+ */
+static PyObject*
+get_process_cpu_affinity(PyObject* self, PyObject* args)
+{
+    unsigned long mask;
+    unsigned int len = sizeof(mask);
+    long pid;
+
+    if (!PyArg_ParseTuple(args, "i", &pid)) {
+        return NULL;
+    }
+    if (sched_getaffinity(pid, len, (cpu_set_t *)&mask) < 0) {
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+    return Py_BuildValue("l", mask);
+}
+
+
+/*
+ * Set process CPU affinity; expects a bitmask
+ */
+static PyObject*
+set_process_cpu_affinity(PyObject* self, PyObject* args)
+{
+    unsigned long mask;
+    unsigned int len = sizeof(mask);
+    long pid;
+
+    if (!PyArg_ParseTuple(args, "ll", &pid, &mask)) {
+        return NULL;
+    }
+    if (sched_setaffinity(pid, len, (cpu_set_t *)&mask)) {
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+
+/*
+ * Return currently connected users as a list of tuples.
+ */
+static PyObject*
+get_system_users(PyObject* self, PyObject* args)
+{
+    PyObject *ret_list = PyList_New(0);
+    PyObject *tuple = NULL;
+    PyObject *user_proc = NULL;
+    struct utmp *ut;
+
+    setutent();
+    while (NULL != (ut = getutent())) {
+        if (ut->ut_type == USER_PROCESS)
+            user_proc = Py_True;
+        else
+            user_proc = Py_False;
+        tuple = Py_BuildValue("(sssfO)",
+            ut->ut_user,              // username
+            ut->ut_line,              // tty
+            ut->ut_host,              // hostname
+            (float)ut->ut_tv.tv_sec,  // tstamp
+            user_proc                 // (bool) user process
+        );
+        PyList_Append(ret_list, tuple);
+        Py_DECREF(tuple);
+    }
+    endutent();
+
+    return ret_list;
+}
+
+
 /*
  * Define the psutil C module methods and initialize the module.
  */
@@ -165,6 +242,12 @@ PsutilMethods[] =
         "device, mount point and filesystem type"},
      {"get_physmem", get_physmem, METH_VARARGS,
         "The physical memory usage as a (total, free, buffer) tuple."},
+     {"get_process_cpu_affinity", get_process_cpu_affinity, METH_VARARGS,
+        "Return process CPU affinity as a Python long (the bitmask)."},
+     {"set_process_cpu_affinity", set_process_cpu_affinity, METH_VARARGS,
+        "Set process CPU affinity; expects a bitmask."},
+     {"get_system_users", get_system_users, METH_VARARGS,
+        "Return currently connected users as a list of tuples"},
 
      {NULL, NULL, 0, NULL}
 };
