@@ -21,6 +21,7 @@
 #include <tlhelp32.h>
 #include <winsock2.h>
 #include <iphlpapi.h>
+#include <wtsapi32.h>
 
 // Link with Iphlpapi.lib
 #pragma comment(lib, "IPHLPAPI.lib")
@@ -293,8 +294,10 @@ static PyObject*
 get_process_create_time(PyObject* self, PyObject* args)
 {
     long        pid;
-    long long unix_time;
+    long long   unix_time;
+    DWORD       exitCode;
     HANDLE      hProcess;
+    BOOL WINAPI ret;
     FILETIME    ftCreate, ftExit, ftKernel, ftUser;
 
     if (! PyArg_ParseTuple(args, "l", &pid)) {
@@ -323,7 +326,26 @@ get_process_create_time(PyObject* self, PyObject* args)
         }
     }
 
+    // Make sure the process is not gone as OpenProcess alone seems to be
+    // unreliable in doing so (it seems a previous call to p.wait() makes
+    // it unreliable).
+    // This check is important as creation time is used to make sure the
+    // process is still running.
+    ret = GetExitCodeProcess(hProcess, &exitCode);
     CloseHandle(hProcess);
+    if (ret != 0) {
+        if (exitCode != STILL_ACTIVE) {
+            return NoSuchProcess();
+        }
+    }
+    else {
+        // Ignore access denied as it means the process is still alive.
+        // For all other errors, we want an exception.
+        if (GetLastError() != ERROR_ACCESS_DENIED) {
+            PyErr_SetFromWindowsErr(0);
+            return NULL;
+        }
+    }
 
     /*
     Convert the FILETIME structure to a Unix time.
@@ -458,6 +480,36 @@ get_process_cmdline(PyObject* self, PyObject* args) {
 }
 
 
+/*
+ * Return process executable path.
+ */
+static PyObject*
+get_process_exe(PyObject* self, PyObject* args) {
+    long pid;
+    HANDLE hProcess;
+    wchar_t exe[MAX_PATH];
+    DWORD nSize = MAX_PATH;
+    DWORD WINAPI ret;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+
+    hProcess = handle_from_pid_waccess(pid, PROCESS_QUERY_INFORMATION);
+    if (NULL == hProcess) {
+        return NULL;
+    }
+
+    if (GetProcessImageFileName(hProcess, &exe, nSize) == 0) {
+        CloseHandle(hProcess);
+        return PyErr_SetFromWindowsErr(0);
+    }
+
+    CloseHandle(hProcess);
+    return Py_BuildValue("s", exe);
+}
+
+
 /*
  * Return the RSS and VMS as a Python tuple.
  */
@@ -636,7 +688,7 @@ get_process_cwd(PyObject* self, PyObject* args)
 
     pebAddress = GetPebAddress(processHandle);
 
-    /* get the address of ProcessParameters */
+    // get the address of ProcessParameters
 #ifdef _WIN64
     if (!ReadProcessMemory(processHandle, (PCHAR)pebAddress + 32,
         &rtlUserProcParamsAddress, sizeof(PVOID), NULL))
@@ -645,22 +697,21 @@ get_process_cwd(PyObject* self, PyObject* args)
         &rtlUserProcParamsAddress, sizeof(PVOID), NULL))
 #endif
     {
-            CloseHandle(processHandle);
-
-            if (GetLastError() == ERROR_PARTIAL_COPY) {
-                /* Usually means the process has gone in the meantime */
-                return NoSuchProcess();
-            }
-            else {
-                return PyErr_SetFromWindowsErr(0);
-            }
+        CloseHandle(processHandle);
 
+        if (GetLastError() == ERROR_PARTIAL_COPY) {
+            // this occurs quite often with system processes
+            return AccessDenied();
+        }
+        else {
+            return PyErr_SetFromWindowsErr(0);
+        }
     }
 
-    /* read the currentDirectory UNICODE_STRING structure.
-       0x24 refers to "CurrentDirectoryPath" of RTL_USER_PROCESS_PARAMETERS
-       structure (http://wj32.wordpress.com/2009/01/24/howto-get-the-command-line-of-processes/)
-     */
+    // Read the currentDirectory UNICODE_STRING structure.
+    // 0x24 refers to "CurrentDirectoryPath" of RTL_USER_PROCESS_PARAMETERS
+    // structure, see:
+    // http://wj32.wordpress.com/2009/01/24/howto-get-the-command-line-of-processes/
 #ifdef _WIN64
     if (!ReadProcessMemory(processHandle, (PCHAR)rtlUserProcParamsAddress + 56,
         &currentDirectory, sizeof(currentDirectory), NULL))
@@ -671,18 +722,18 @@ get_process_cwd(PyObject* self, PyObject* args)
     {
         CloseHandle(processHandle);
         if (GetLastError() == ERROR_PARTIAL_COPY) {
-            /* Usually means the process has gone in the meantime */
-            return NoSuchProcess();
+            // this occurs quite often with system processes
+            return AccessDenied();
         }
         else {
             return PyErr_SetFromWindowsErr(0);
         }
     }
 
-    /* allocate memory to hold the command line */
+    // allocate memory to hold cwd
     currentDirectoryContent = (WCHAR *)malloc(currentDirectory.Length+1);
 
-    /* read the command line */
+    // read cwd
     if (!ReadProcessMemory(processHandle, currentDirectory.Buffer,
         currentDirectoryContent, currentDirectory.Length, NULL))
     {
@@ -690,16 +741,17 @@ get_process_cwd(PyObject* self, PyObject* args)
         free(currentDirectoryContent);
 
         if (GetLastError() == ERROR_PARTIAL_COPY) {
-            /* Usually means the process has gone in the meantime */
-            return NoSuchProcess();
+            // this occurs quite often with system processes
+            return AccessDenied();
         }
         else {
             return PyErr_SetFromWindowsErr(0);
         }
     }
 
-    // null-terminate the string to prevent wcslen from returning incorrect length
-    // the length specifier is in characters, but commandLine.Length is in bytes
+    // null-terminate the string to prevent wcslen from returning
+    // incorrect length the length specifier is in characters, but
+    // currentDirectory.Length is in bytes
     currentDirectoryContent[(currentDirectory.Length/sizeof(WCHAR))] = '\0';
 
     // convert wchar array to a Python unicode string, and then to UTF8
@@ -712,7 +764,8 @@ get_process_cwd(PyObject* self, PyObject* args)
         cwd = PyUnicode_AsUTF8String(cwd_from_wchar);
     #endif
 
-    // decrement the reference count on our temp unicode str to avoid mem leak
+    // decrement the reference count on our temp unicode str to avoid
+    // mem leak
     Py_XDECREF(cwd_from_wchar);
     returnPyObj = Py_BuildValue("N", cwd);
 
@@ -1302,6 +1355,8 @@ get_process_connections(PyObject* self, PyObject* args)
                                                         "GetExtendedTcpTable");
         getExtendedUdpTable = (_GetExtendedUdpTable)GetProcAddress(iphlpapi,
                                                         "GetExtendedUdpTable");
+        FreeLibrary(ntdll);
+        FreeLibrary(iphlpapi);
     }
 
     if ((getExtendedTcpTable == NULL) || (getExtendedUdpTable == NULL)) {
@@ -1638,9 +1693,6 @@ get_process_io_counters(PyObject* self, PyObject* args)
     if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
     }
-    if (pid == 0) {
-        return AccessDenied();
-    }
     hProcess = handle_from_pid(pid);
     if (NULL == hProcess) {
         return NULL;
@@ -1657,6 +1709,64 @@ get_process_io_counters(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return process CPU affinity as a bitmask
+ */
+static PyObject*
+get_process_cpu_affinity(PyObject* self, PyObject* args)
+{
+    DWORD pid;
+    HANDLE hProcess;
+    PDWORD_PTR proc_mask;
+    PDWORD_PTR system_mask;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    hProcess = handle_from_pid(pid);
+    if (hProcess == NULL) {
+        return NULL;
+    }
+    if (GetProcessAffinityMask(hProcess, &proc_mask, &system_mask) == 0) {
+        CloseHandle(hProcess);
+        return PyErr_SetFromWindowsErr(0);
+    }
+
+    CloseHandle(hProcess);
+    return Py_BuildValue("k", (unsigned long)proc_mask);
+}
+
+
+/*
+ * Set process CPU affinity
+ */
+static PyObject*
+set_process_cpu_affinity(PyObject* self, PyObject* args)
+{
+    DWORD pid;
+    HANDLE hProcess;
+    DWORD dwDesiredAccess = PROCESS_QUERY_INFORMATION | PROCESS_SET_INFORMATION;
+    DWORD_PTR mask;
+
+    if (! PyArg_ParseTuple(args, "lk", &pid, &mask)) {
+        return NULL;
+    }
+    hProcess = handle_from_pid_waccess(pid, dwDesiredAccess);
+    if (hProcess == NULL) {
+        return NULL;
+    }
+
+    if (SetProcessAffinityMask(hProcess, mask) == 0) {
+        CloseHandle(hProcess);
+        return PyErr_SetFromWindowsErr(0);
+    }
+
+    CloseHandle(hProcess);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+
 /*
  * Return True if one of the process threads is in a waiting or
  * suspended status.
@@ -1845,8 +1955,10 @@ get_disk_io_counters(PyObject* self, PyObject* args)
             break;
         }
 
-        if (DeviceIoControl (hDevice, IOCTL_DISK_PERFORMANCE, NULL, 0,
-            &diskPerformance, sizeof (DISK_PERFORMANCE), &dwSize, NULL)) {
+        if (DeviceIoControl(hDevice, IOCTL_DISK_PERFORMANCE, NULL, 0,
+                            &diskPerformance, sizeof(DISK_PERFORMANCE),
+                            &dwSize, NULL))
+        {
             sprintf (szDeviceDisplay, "PhysicalDrive%d", devNum);
             py_disk_info = Py_BuildValue("(IILLLL)",
                                          diskPerformance.ReadCount,
@@ -1862,6 +1974,12 @@ get_disk_io_counters(PyObject* self, PyObject* args)
                                  py_disk_info);
             Py_XDECREF(py_disk_info);
         }
+        else {
+            // XXX we might get here with ERROR_INSUFFICIENT_BUFFER when
+            // compiling with mingw32; not sure what to do.
+            //return PyErr_SetFromWindowsErr(0);
+            ;;
+        }
 
         CloseHandle(hDevice);
     }
@@ -1870,17 +1988,55 @@ get_disk_io_counters(PyObject* self, PyObject* args)
 }
 
 
+static char *get_drive_type(int type)
+{
+    switch (type) {
+        case DRIVE_FIXED:
+            return "fixed";
+        case DRIVE_CDROM:
+            return "cdrom";
+        case DRIVE_REMOVABLE:
+            return "removable";
+        case DRIVE_UNKNOWN:
+            return "unknown";
+        case DRIVE_NO_ROOT_DIR:
+            return "unmounted";
+        case DRIVE_REMOTE:
+            return "remote";
+        case DRIVE_RAMDISK:
+            return "ramdisk";
+        default:
+            return "?";
+    }
+}
+
+
+#define _ARRAYSIZE(a) (sizeof(a)/sizeof(a[0]))
+
 /*
- * Return disk partitions as a list of strings.
+ * Return disk partitions as a list of tuples such as
+ * (drive_letter, drive_letter, type, "")
  */
 static PyObject*
-win32_GetLogicalDriveStrings(PyObject* self, PyObject* args)
+get_disk_partitions(PyObject* self, PyObject* args)
 {
     DWORD num_bytes;
     char drive_strings[255];
     char* drive_letter = drive_strings;
+    int all;
+    int type;
+    int ret;
+    char opts[20];
+    LPTSTR fs_type[MAX_PATH + 1] = { 0 };
+    DWORD pflags = 0;
+    PyObject* py_all;
     PyObject* py_retlist = PyList_New(0);
-    PyObject* py_string = NULL;
+    PyObject* py_tuple = NULL;
+
+    if (! PyArg_ParseTuple(args, "O", &py_all)) {
+        return NULL;
+    }
+    all = PyObject_IsTrue(py_all);
 
     Py_BEGIN_ALLOW_THREADS
     num_bytes = GetLogicalDriveStrings(254, drive_letter);
@@ -1891,61 +2047,317 @@ win32_GetLogicalDriveStrings(PyObject* self, PyObject* args)
     }
 
     while (*drive_letter != 0) {
-        py_string = Py_BuildValue("s", drive_letter);
-        PyList_Append(py_retlist, py_string);
-        Py_DECREF(py_string);
-        drive_letter = strchr(drive_letter, 0) + 1;
+        opts[0] = 0;
+        fs_type[0] = 0;
+
+        Py_BEGIN_ALLOW_THREADS
+        type = GetDriveType(drive_letter);
+        Py_END_ALLOW_THREADS
+
+        // by default we only show hard drives and cd-roms
+        if (all == 0) {
+            if ((type == DRIVE_UNKNOWN) ||
+                (type == DRIVE_NO_ROOT_DIR) ||
+                (type == DRIVE_REMOTE) ||
+                (type == DRIVE_RAMDISK)) {
+                goto next;
+            }
+            // floppy disk: skip it by default as it introduces a
+            // considerable slowdown.
+            if ((type == DRIVE_REMOVABLE) && (strcmp(drive_letter, "A:\\") == 0)) {
+                goto next;
+            }
+        }
+
+        ret = GetVolumeInformation(drive_letter, NULL, _ARRAYSIZE(drive_letter),
+                                   NULL, NULL, &pflags, fs_type,
+                                   _ARRAYSIZE(fs_type));
+        if (ret == 0) {
+            // We might get here in case of a floppy hard drive, in
+            // which case the error is (21, "device not ready").
+            // Let's pretend it didn't happen as we already have
+            // the drive name and type ('removable').
+            strcat(opts, "");
+            SetLastError(0);
+        }
+        else {
+            if (pflags & FILE_READ_ONLY_VOLUME) {
+                strcat(opts, "ro");
+            }
+            else {
+                strcat(opts, "rw");
+            }
+            if (pflags & FILE_VOLUME_IS_COMPRESSED) {
+                strcat(opts, ",compressed");
+            }
+        }
+
+        if (strlen(opts) > 0) {
+            strcat(opts, ",");
+        }
+        strcat(opts, get_drive_type(type));
+
+        py_tuple = Py_BuildValue("(ssss)",
+            drive_letter,
+            drive_letter,
+            fs_type,  // either FAT, FAT32, NTFS, HPFS, CDFS, UDF or NWFS
+            opts);
+        PyList_Append(py_retlist, py_tuple);
+        Py_DECREF(py_tuple);
+        goto next;
+
+        next:
+            drive_letter = strchr(drive_letter, 0) + 1;
     }
 
     return py_retlist;
 }
 
 
-static PyObject*
-win32_GetDriveType(PyObject* self, PyObject* args)
-{
-    LPCTSTR drive_letter;
-    int type;
-    char* type_str;
+#ifdef UNICODE
+#define WTSOpenServer WTSOpenServerW
+#else
+#define WTSOpenServer WTSOpenServerA
+#endif
 
-    if (! PyArg_ParseTuple(args, "s", &drive_letter)) {
+
+/*
+ * Return a Python dict of tuples for disk I/O information
+ */
+static PyObject*
+get_system_users(PyObject* self, PyObject* args)
+{
+    PyObject* py_retlist = PyList_New(0);
+    PyObject* py_tuple = NULL;
+    PyObject* py_address = NULL;
+
+    HANDLE hServer = NULL;
+    LPTSTR buffer_user = NULL;
+    LPTSTR buffer_addr = NULL;
+    PWTS_SESSION_INFO sessions = NULL;
+    DWORD count;
+    DWORD i;
+    DWORD sessionId;
+    DWORD bytes;
+    PWTS_CLIENT_ADDRESS address;
+    char address_str[50];
+    long long unix_time;
+
+    PWINSTATIONQUERYINFORMATIONW WinStationQueryInformationW;
+    WINSTATION_INFO station_info;
+    HINSTANCE hInstWinSta = NULL;
+    ULONG returnLen;
+
+    hInstWinSta = LoadLibraryA("winsta.dll");
+    WinStationQueryInformationW = (PWINSTATIONQUERYINFORMATIONW)
+        GetProcAddress(hInstWinSta, "WinStationQueryInformationW");
+
+    hServer = WTSOpenServer('\0');
+    if (hServer == NULL) {
+        goto error;
+    }
+
+    if (WTSEnumerateSessions(hServer, 0, 1, &sessions, &count) == 0) {
+        goto error;
+    }
+
+    for (i=0; i<count; i++) {
+        sessionId = sessions[i].SessionId;
+        if (buffer_user != NULL) {
+            WTSFreeMemory(buffer_user);
+        }
+        if (buffer_addr != NULL) {
+            WTSFreeMemory(buffer_addr);
+        }
+
+        buffer_user = NULL;
+        buffer_addr = NULL;
+
+        // username
+        bytes = 0;
+        if (WTSQuerySessionInformation(hServer, sessionId, WTSUserName,
+                                        &buffer_user, &bytes) == 0) {
+            goto error;
+        }
+        if (bytes == 1) {
+            continue;
+        }
+
+        // address
+        bytes = 0;
+        if (WTSQuerySessionInformation(hServer, sessionId, WTSClientAddress,
+                                       &buffer_addr, &bytes) == 0) {
+            goto error;
+        }
+
+        address = (PWTS_CLIENT_ADDRESS)buffer_addr;
+        if (address->AddressFamily == 0) {  // AF_INET
+            sprintf(address_str, "%u.%u.%u.%u", address->Address[0],
+                                                address->Address[1],
+                                                address->Address[2],
+                                                address->Address[3]);
+            py_address = Py_BuildValue("s", address_str);
+        }
+        else {
+            py_address = Py_None;
+        }
+
+        // login time
+        if (!WinStationQueryInformationW(hServer,
+                                         sessionId,
+                                         WinStationInformation,
+                                         &station_info,
+                                         sizeof(station_info),
+                                         &returnLen))
+                                         {
+            goto error;
+        }
+
+        unix_time = ((LONGLONG)station_info.ConnectTime.dwHighDateTime) << 32;
+        unix_time += station_info.ConnectTime.dwLowDateTime - 116444736000000000LL;
+        unix_time /= 10000000;
+
+        py_tuple = Py_BuildValue("sOd", buffer_user,
+                                        py_address,
+                                        (double)unix_time);
+        PyList_Append(py_retlist, py_tuple);
+        Py_XDECREF(py_address);
+        Py_XDECREF(py_tuple);
+    }
+
+    WTSCloseServer(hServer);
+    WTSFreeMemory(sessions);
+    WTSFreeMemory(buffer_user);
+    WTSFreeMemory(buffer_addr);
+    FreeLibrary(hInstWinSta);
+    return py_retlist;
+
+error:
+    if (hInstWinSta != NULL) {
+        FreeLibrary(hInstWinSta);
+    }
+    if (hServer != NULL) {
+        WTSCloseServer(hServer);
+    }
+    if (sessions != NULL) {
+        WTSFreeMemory(sessions);
+    }
+    if (buffer_user != NULL) {
+        WTSFreeMemory(buffer_user);
+    }
+    if (buffer_addr != NULL) {
+        WTSFreeMemory(buffer_addr);
+    }
+    return PyErr_SetFromWindowsErr(0);
+}
+
+
+/*
+ * Return the number of handles opened by process.
+ */
+static PyObject*
+get_process_num_handles(PyObject* self, PyObject* args)
+{
+    DWORD pid;
+    HANDLE hProcess;
+    DWORD handleCount;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    hProcess = handle_from_pid(pid);
+    if (NULL == hProcess) {
+        return NULL;
+    }
+    if (! GetProcessHandleCount(hProcess, &handleCount)) {
+        CloseHandle(hProcess);
+        return PyErr_SetFromWindowsErr(0);
+    }
+    CloseHandle(hProcess);
+    return Py_BuildValue("k", handleCount);
+}
+
+
+static char *get_region_protection_string(ULONG protection)
+{
+    switch (protection & 0xff) {
+        case PAGE_NOACCESS:
+            return "";
+        case PAGE_READONLY:
+            return "r";
+        case PAGE_READWRITE:
+            return "rw";
+        case PAGE_WRITECOPY:
+            return "wc";
+        case PAGE_EXECUTE:
+            return "x";
+        case PAGE_EXECUTE_READ:
+            return "xr";
+        case PAGE_EXECUTE_READWRITE:
+            return "xrw";
+        case PAGE_EXECUTE_WRITECOPY:
+            return "xwc";
+        default:
+            return "?";
+    }
+}
+
+/*
+ * Return a list of process's memory mappings.
+ */
+static PyObject*
+get_process_memory_maps(PyObject* self, PyObject* args)
+{
+    DWORD pid;
+    HANDLE hProcess;
+    MEMORY_BASIC_INFORMATION basicInfo;
+    PVOID baseAddress;
+    PVOID previousAllocationBase;
+    CHAR mappedFileName[MAX_PATH];
+    SYSTEM_INFO system_info;
+    LPVOID maxAddr;
+    PyObject* py_list = PyList_New(0);
+    PyObject* py_tuple;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    hProcess = handle_from_pid(pid);
+    if (NULL == hProcess) {
         return NULL;
     }
 
-    Py_BEGIN_ALLOW_THREADS
-    type = GetDriveType(drive_letter);
-    Py_END_ALLOW_THREADS
+    GetSystemInfo(&system_info);
+    maxAddr = system_info.lpMaximumApplicationAddress;
+    baseAddress = NULL;
+    previousAllocationBase = NULL;
 
-    switch (type) {
-        case DRIVE_UNKNOWN:
-            type_str = "unknown";
-            break;
-        case DRIVE_NO_ROOT_DIR:
-            type_str = "unmounted";
-        case DRIVE_REMOVABLE:
-            type_str = "removable";
-            break;
-        case DRIVE_FIXED:
-            type_str = "fixed";
-            break;
-        case DRIVE_REMOTE:
-            type_str = "remote";
-            break;
-        case DRIVE_CDROM:
-            type_str = "cdrom";
-            break;
-        case DRIVE_RAMDISK:
-            type_str = "ramdisk";
-            break;
-        default:
-            type_str = "?";
+    while (VirtualQueryEx(hProcess, baseAddress, &basicInfo,
+                          sizeof(MEMORY_BASIC_INFORMATION)))
+    {
+        if (baseAddress > maxAddr) {
             break;
+        }
+        if (GetMappedFileNameA(hProcess, baseAddress, mappedFileName,
+                               sizeof(mappedFileName)))
+        {
+            py_tuple = Py_BuildValue("(nssI)",
+                (ULONG_PTR)baseAddress,
+                get_region_protection_string(basicInfo.Protect),
+                mappedFileName,
+                basicInfo.RegionSize
+            );
+            PyList_Append(py_list, py_tuple);
+            Py_DECREF(py_tuple);
+        }
+        previousAllocationBase = basicInfo.AllocationBase;
+        baseAddress = (PCHAR)baseAddress + basicInfo.RegionSize;
     }
 
-    return Py_BuildValue("s", type_str);
+    CloseHandle(hProcess);
+    return py_list;
 }
 
-
 // ------------------------ Python init ---------------------------
 
 static PyMethodDef
@@ -1953,21 +2365,23 @@ PsutilMethods[] =
 {
     // --- per-process functions
 
-     {"get_process_name", get_process_name, METH_VARARGS,
+    {"get_process_name", get_process_name, METH_VARARGS,
         "Return process name"},
-     {"get_process_cmdline", get_process_cmdline, METH_VARARGS,
+    {"get_process_cmdline", get_process_cmdline, METH_VARARGS,
         "Return process cmdline as a list of cmdline arguments"},
-     {"get_process_ppid", get_process_ppid, METH_VARARGS,
+    {"get_process_exe", get_process_exe, METH_VARARGS,
+        "Return path of the process executable"},
+    {"get_process_ppid", get_process_ppid, METH_VARARGS,
         "Return process ppid as an integer"},
-     {"kill_process", kill_process, METH_VARARGS,
-         "Kill the process identified by the given PID"},
-     {"get_process_cpu_times", get_process_cpu_times, METH_VARARGS,
+    {"kill_process", kill_process, METH_VARARGS,
+        "Kill the process identified by the given PID"},
+    {"get_process_cpu_times", get_process_cpu_times, METH_VARARGS,
         "Return tuple of user/kern time for the given PID"},
-     {"get_process_create_time", get_process_create_time, METH_VARARGS,
-         "Return a float indicating the process create time expressed in "
-         "seconds since the epoch"},
-     {"get_memory_info", get_memory_info, METH_VARARGS,
-         "Return a tuple of RSS/VMS memory information"},
+    {"get_process_create_time", get_process_create_time, METH_VARARGS,
+        "Return a float indicating the process create time expressed in "
+        "seconds since the epoch"},
+    {"get_memory_info", get_memory_info, METH_VARARGS,
+        "Return a tuple of RSS/VMS memory information"},
     {"get_process_cwd", get_process_cwd, METH_VARARGS,
         "Return process current working directory"},
     {"suspend_process", suspend_process, METH_VARARGS,
@@ -1990,41 +2404,50 @@ PsutilMethods[] =
         "Return process priority."},
     {"set_process_priority", set_process_priority, METH_VARARGS,
         "Set process priority."},
+    {"get_process_cpu_affinity", get_process_cpu_affinity, METH_VARARGS,
+        "Return process CPU affinity as a bitmask."},
+    {"set_process_cpu_affinity", set_process_cpu_affinity, METH_VARARGS,
+        "Set process CPU affinity."},
     {"get_process_io_counters", get_process_io_counters, METH_VARARGS,
         "Get process I/O counters."},
     {"is_process_suspended", is_process_suspended, METH_VARARGS,
         "Return True if one of the process threads is in a suspended state"},
+    {"get_process_num_handles", get_process_num_handles, METH_VARARGS,
+        "Return the number of handles opened by process."},
+    {"get_process_memory_maps", get_process_memory_maps, METH_VARARGS,
+        "Return a list of process's memory mappings"},
 
     // --- system-related functions
 
-     {"get_pid_list", get_pid_list, METH_VARARGS,
+    {"get_pid_list", get_pid_list, METH_VARARGS,
         "Returns a list of PIDs currently running on the system"},
-     {"pid_exists", pid_exists, METH_VARARGS,
-         "Determine if the process exists in the current process list."},
-     {"get_num_cpus", get_num_cpus, METH_VARARGS,
-         "Returns the number of CPUs on the system"},
-     {"get_system_uptime", get_system_uptime, METH_VARARGS,
-         "Return system uptime"},
-     {"get_system_phymem", get_system_phymem, METH_VARARGS,
-         "Return the total amount of physical memory, in bytes"},
-     {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
-         "Return system per-cpu times as a list of tuples"},
-     {"get_disk_usage", get_disk_usage, METH_VARARGS,
-         "Return path's disk total and free as a Python tuple."},
-     {"get_network_io_counters", get_network_io_counters, METH_VARARGS,
-         "Return dict of tuples of networks I/O information."},
-     {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
+    {"pid_exists", pid_exists, METH_VARARGS,
+        "Determine if the process exists in the current process list."},
+    {"get_num_cpus", get_num_cpus, METH_VARARGS,
+        "Returns the number of CPUs on the system"},
+    {"get_system_uptime", get_system_uptime, METH_VARARGS,
+        "Return system uptime"},
+    {"get_system_phymem", get_system_phymem, METH_VARARGS,
+        "Return the total amount of physical memory, in bytes"},
+    {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
+        "Return system per-cpu times as a list of tuples"},
+    {"get_disk_usage", get_disk_usage, METH_VARARGS,
+        "Return path's disk total and free as a Python tuple."},
+    {"get_network_io_counters", get_network_io_counters, METH_VARARGS,
+        "Return dict of tuples of networks I/O information."},
+    {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
          "Return dict of tuples of disks I/O information."},
+    {"get_system_users", get_system_users, METH_VARARGS,
+        "Return a list of currently connected users."},
+    {"get_disk_partitions", get_disk_partitions, METH_VARARGS,
+        "Return disk partitions."},
+
 
      // --- windows API bindings
-     {"win32_GetLogicalDriveStrings", win32_GetLogicalDriveStrings, METH_VARARGS,
-         "GetLogicalDriveStrings binding"},
-     {"win32_GetDriveType", win32_GetDriveType, METH_VARARGS,
-         "GetDriveType binding"},
-     {"win32_QueryDosDevice", win32_QueryDosDevice, METH_VARARGS,
-         "QueryDosDevice binding"},
+    {"win32_QueryDosDevice", win32_QueryDosDevice, METH_VARARGS,
+        "QueryDosDevice binding"},
 
-     {NULL, NULL, 0, NULL}
+    {NULL, NULL, 0, NULL}
 };
 
 
