@@ -16,6 +16,7 @@
 static PyObject* kill_process(PyObject* self, PyObject* args);
 static PyObject* get_process_name(PyObject* self, PyObject* args);
 static PyObject* get_process_cmdline(PyObject* self, PyObject* args);
+static PyObject* get_process_exe(PyObject* self, PyObject* args);
 static PyObject* get_process_ppid(PyObject* self, PyObject* args);
 static PyObject* get_process_cpu_times(PyObject* self, PyObject* args);
 static PyObject* get_process_create_time(PyObject* self, PyObject* args);
@@ -33,6 +34,10 @@ static PyObject* get_process_priority(PyObject* self, PyObject* args);
 static PyObject* set_process_priority(PyObject* self, PyObject* args);
 static PyObject* get_process_io_counters(PyObject* self, PyObject* args);
 static PyObject* is_process_suspended(PyObject* self, PyObject* args);
+static PyObject* get_process_cpu_affinity(PyObject* self, PyObject* args);
+static PyObject* set_process_cpu_affinity(PyObject* self, PyObject* args);
+static PyObject* get_process_num_handles(PyObject* self, PyObject* args);
+static PyObject* get_process_memory_maps(PyObject* self, PyObject* args);
 
 // --- system-related functions
 
@@ -46,12 +51,11 @@ static PyObject* get_disk_usage(PyObject* self, PyObject* args);
 static PyObject* get_disk_partitions(PyObject* self, PyObject* args);
 static PyObject* get_network_io_counters(PyObject* self, PyObject* args);
 static PyObject* get_disk_io_counters(PyObject* self, PyObject* args);
+static PyObject* get_system_users(PyObject* self, PyObject* args);
 
 // --- windows API bindings
 
 static PyObject* win32_QueryDosDevice(PyObject* self, PyObject* args);
-static PyObject* win32_GetLogicalDriveStrings(PyObject* self, PyObject* args);
-static PyObject* win32_GetDriveType(PyObject* self, PyObject* args);
 
 // --- internal
 int suspend_resume_process(DWORD pid, int suspend);
