@@ -8,11 +8,15 @@
 
 """Linux specific tests.  These are implicitly run by test_psutil.py."""
 
+from __future__ import division
 import unittest
 import subprocess
 import sys
+import time
+import os
 
-from test_psutil import sh
+from test_psutil import sh, get_test_subprocess
+from psutil._compat import PY3
 import psutil
 
 
@@ -23,7 +27,7 @@ class LinuxSpecificTestCase(unittest.TestCase):
         # command line utility
         p = subprocess.Popen("free", shell=1, stdout=subprocess.PIPE)
         output = p.communicate()[0].strip()
-        if sys.version_info >= (3,):
+        if PY3:
             output = str(output, sys.stdout.encoding)
         free_cmem = int(output.split('\n')[1].split()[6])
         psutil_cmem = psutil.cached_phymem() / 1024
@@ -34,7 +38,7 @@ class LinuxSpecificTestCase(unittest.TestCase):
         # command line utility
         p = subprocess.Popen("free", shell=1, stdout=subprocess.PIPE)
         output = p.communicate()[0].strip()
-        if sys.version_info >= (3,):
+        if PY3:
             output = str(output, sys.stdout.encoding)
         free_cmem = int(output.split('\n')[1].split()[5])
         psutil_cmem = psutil.phymem_buffers() / 1024
@@ -61,10 +65,26 @@ class LinuxSpecificTestCase(unittest.TestCase):
             self.assertEqual(usage.total, total)
             # 10 MB tollerance
             if abs(usage.free - free) > 10 * 1024 * 1024:
-                self.fail("psutil=%s, df=%s" % usage.free, free)
+                self.fail("psutil=%s, df=%s" % (usage.free, free))
             if abs(usage.used - used) > 10 * 1024 * 1024:
-                self.fail("psutil=%s, df=%s" % usage.used, used)
+                self.fail("psutil=%s, df=%s" % (usage.used, used))
 
+    def test_memory_maps(self):
+        sproc = get_test_subprocess()
+        time.sleep(1)
+        p = psutil.Process(sproc.pid)
+        maps = p.get_memory_maps(grouped=False)
+        pmap = sh('pmap -x %s' % p.pid).split('\n')
+        del pmap[0]; del pmap[0]  # get rid of header
+        while maps and pmap:
+            this = maps.pop(0)
+            other = pmap.pop(0)
+            addr, _, rss, dirty, mode, path = other.split(None, 5)
+            if not path.startswith('[') and not path.endswith(']'):
+                self.assertEqual(path, os.path.basename(this.path))
+            self.assertEqual(int(rss) * 1024, this.rss)
+            # test only rwx chars, ignore 's' and 'p'
+            self.assertEqual(mode[:3], this.perms[:3])
 
 if __name__ == '__main__':
     test_suite = unittest.TestSuite()
