@@ -19,19 +19,13 @@ import time
 import socket
 
 import psutil
+from psutil._compat import PY3, callable, xrange
 from test_psutil import reap_children, skipUnless, skipIf, supports_ipv6, \
-                        POSIX, LINUX, WINDOWS, OSX, BSD, PY3
+                        POSIX, LINUX, WINDOWS, OSX, BSD
 
 LOOPS = 1000
 TOLERANCE = 4096
 
-if PY3:
-    xrange = range
-try:
-    callable
-except NameError:
-    callable = lambda x: hasattr(x, '__call__')
-
 
 class Base(unittest.TestCase):
 
@@ -77,7 +71,7 @@ class Base(unittest.TestCase):
     def get_mem():
         return psutil.Process(os.getpid()).get_memory_info()[0]
 
-    def call(self):
+    def call(self, *args, **kwargs):
         raise NotImplementedError("must be implemented in subclass")
 
 
@@ -150,9 +144,27 @@ class TestProcessObjectLeaks(Base):
     def test_getcwd(self):
         self.execute('getcwd')
 
+    @skipUnless(WINDOWS)
+    def test_getcwd(self):
+        self.execute('get_num_handles')
+
+    @skipUnless(LINUX or WINDOWS)
+    def test_get_cpu_affinity(self):
+        self.execute('get_cpu_affinity')
+
+    @skipUnless(LINUX or WINDOWS)
+    def test_get_cpu_affinity(self):
+        affinity = psutil.Process(os.getpid()).get_cpu_affinity()
+        self.execute('set_cpu_affinity', affinity)
+
     def test_get_open_files(self):
         self.execute('get_open_files')
 
+    # OSX implementation is unbelievably slow
+    @skipIf(OSX)
+    def test_get_memory_maps(self):
+        self.execute('get_memory_maps')
+
     # XXX - BSD still uses provisional lsof implementation
     # Linux implementation is pure python so since it's slow we skip it
     @skipIf(BSD or LINUX)
@@ -223,6 +235,12 @@ class TestModuleFunctionsLeaks(Base):
     def test_disk_io_counters(self):
         self.execute('disk_io_counters')
 
+    # XXX - on Windows this produces a false positive
+    @skipIf(WINDOWS)
+    def test_get_users(self):
+        self.execute('get_users')
+
+
 def test_main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(TestProcessObjectLeaks))
